#pragma once
/*****************************************************************//**
 * \file   SARSatelliteSimulationAbstractCls.h
 * \brief  SARǷеóģ,һЩõĽṹ
 *	Ҫ
 * 
 * 1. ״̬
 * 2. ָ״̬ɨǡָ
 * 3. ߷ͼ ָΪ0ȣZ->Y  theta , X->Y phi )
 * 4. ǲ(Ƶʡʱ䡢PRFFs)
 * ߷ͼ  phi=0  Xзһ£ʱ G(phi)>G(theta)  G(V)>G(H) ˮƽ 
 * ߷ͼ  phi=90  yзһ£ʱ G(phi)<G(theta)  G(V)<G(H) ֱ
 * \author 
 * \date   October 2024
 *********************************************************************/
#include "BaseConstVariable.h"
#include "LogInfoCls.h"
#include <QString>
#include <vector>
#include <Eigen/Core>
#include <Eigen/Dense>
#include "GeoOperator.h"

 
/// <summary>
/// ߷ͼĻȡ
/// עʹ ˫Բֵ
/// </summary>
class AbstractRadiationPattern {
public:
	AbstractRadiationPattern();
	virtual ~AbstractRadiationPattern();
public:
	virtual double getGain(double theta, double phi);
	virtual std::vector<RadiationPatternGainPoint> getGainList();
	virtual ErrorCode getGain(double& theta, double& phi, double& GainValue);
	virtual ErrorCode getGainLinear(double& theta, double& phi, double& GainValue);
	double getGainLearThetaPhi(double theta, double phi);
	virtual ErrorCode setGain(double theta, double phi, double GainValue);
	virtual ErrorCode RecontructGainMatrix(double threshold=-3);
	virtual std::vector<double> getThetas();
	virtual std::vector<double> getPhis();
	virtual Eigen::MatrixXd getGainMatrix();
	virtual double getMaxTheta();
	virtual double getMinTheta();
	virtual double getMaxPhi();
	virtual double getMinPhi();


private:
	std::vector<RadiationPatternGainPoint> GainMap; // ߷ͼ
	std::vector<double> thetas;
	std::vector<double> phis;
	Eigen::MatrixXd GainMatrix;
	double maxGain, maxThetaGain, maxPhiGain;
	double mintheta, maxtheta, minphi, maxphi;
	double EdgethetaMin, EdgethetaMax;
	double EdgeXMin, EdgeXMax;
};




/// <summary>
/// ǹģ̬ͣ
/// ر˵нǶȶǻ
/// ûر˵ϵοGF3:
/// չŷΪXᣬ
/// XᶨΪ ߰ڶΪ0ʱǷз
/// ZᶨΪ ZӦΪָ
/// H Ϊ ֱZ->X ƽ
/// V Ϊ ˮƽZ->Y ƽ
///			     -Z
///				 |
///  [=====]X<---Y---[======]    
///				 _
///				___
///			   _____   
///          _________
///		
/// 
/// 
///			   -Z	
///			   ^	   
///		       |
///		X<-Fly-Y
///		       _
///		      ___
///		    _______
/// 
/// 
///           (+				        ӣ-
///      Z				  Z					            Z		
/// 	 ^				   \						   /
/// 	 |					\						  /
///  x<--O---			 x<--O---				 x<--O---
///      |				     |\    				    /|    
/// 						   \				   /  	
/// 
/// 
///   X=Y x Z          ڶ(-				    ڶǣ+
///      Z				  Z					            Z		
/// 	 ^				   \						   /
/// 	 |					\						  /
///  y<--X---			 y<--X---				 y<--X---
///      |				     |\    				    /|    
/// 						   \				   /  	
///  עʱ䰴աϳɿ׾״㷨ʵ֣cummingУǰΪ - Ϊ + 
/// 
/// </summary>
class AbstractSatelliteOribtModel {

public:
	virtual ~AbstractSatelliteOribtModel();

public: // ǹڵ
	virtual SatelliteOribtNode getSatelliteOribtNode(double& timeFromStartTime,bool& antAzAngleFlag); // ȡڵ
	virtual ErrorCode getSatelliteOribtNode(double& timeFromStartTime, SatelliteOribtNode& node, bool& antAzAngleFlag); // ȡڵ
	virtual ErrorCode getSatelliteAntDirectNormal(SatelliteOribtNode& Rs, Vector3D& Rt, SatelliteAntDirect& antNode); // Ŀ߷ͼеλ

public: // ǹģ͵Ĳοʱڵ
	virtual void setSatelliteOribtStartTime(long double oribtStartTime);// ģͲοʱ
	virtual long double  getSatelliteOribtStartTime();// ȡģͲοʱ
public: // ز
	virtual void setbeamAngle(double beamAngle,bool RightLook);// òλ

public: // ߰ڶز
	virtual void setAzAngleRange(double cycletime,double minAzAngle,double maxAzAngle,double referenceAzAngle, double referenceTimeFromStartTime);//λǱ任ѭʱ ;λǱ任Χ ضʱķλǣڼ㷽λǱ仯
	virtual double getAzAngleInCurrentTimeFromStartTime(double& currentTime);// ȡǰʱ ߰ڶ
	virtual ErrorCode getAzAngleInCurrentTimeFromStartTime(double& currentTime, double& AzAngle);// ȡǰʱ ߰ڶ
public: // ԭʼµϵĵȻ-Z Ϊ ָ 
	virtual void setAntnnaAxisX(double X,double Y,double Z); // Xָ 
	virtual void setAntnnaAxisY(double X, double Y, double Z); // Xָ
	virtual void setAntnnaAxisZ(double X, double Y, double Z); // Xָ
	virtual ErrorCode getZeroDopplerAntDirect(SatelliteOribtNode& node);// ȡ0߷
	virtual ErrorCode getAntnnaDirection( SatelliteOribtNode &node); // ȡָ
public:
	virtual double getPt();
	virtual double getGri();
	virtual void setPt(double Pt);
	virtual void setGri(double gri);
};




/// <summary>
/// SARǵϢ
/// 1. ϵͳΪ WGS84굥λ ףʱ䵥λǣ 
/// 2. עһ˵PRFͼأFs  ͼص
/// 3. 
/// </summary>
class AbstractSARSatelliteModel
{
public: // ģ
	virtual SatelliteOribtNode getSatelliteOribtNode(double& timeFromStartTime, bool& antAzAngleFlag);// ùڵ㣬
	virtual ErrorCode getSatelliteOribtNode(double& timeFromStartTime, SatelliteOribtNode& node, bool& antAzAngleFlag); // ȡڵ
	virtual ErrorCode getSatelliteAntDirectNormal(SatelliteOribtNode& Rs, Vector3D& Rt, SatelliteAntDirect& antNode); // Ŀ߷ͼеλ
	virtual void	 setSatelliteOribtModel(std::shared_ptr<AbstractSatelliteOribtModel>  model); // ùģ
public:// Ӱʱ
	virtual void setSARImageStartTime(long double imageStartTime);// ʼʱ
	virtual void setSARImageEndTime(long double imageEndTime);	  // ʱ
	
	virtual double getSARImageStartTime();
	virtual double getSARImageEndTime();

	virtual double getNearRange(); // б -- ʱ
	virtual void   setNearRange(double NearRange);

	virtual double getFarRange(); // Զб 
	virtual void   setFarRange(double FarRange);

	virtual bool   getIsRightLook() ; // Ƿ
	virtual void   setIsRightLook(bool isR) ;
public: // 
	virtual void	setCenterFreq(double Freq); // Ƶ
	virtual double	getCenterFreq();

	virtual void	setCenterLamda(double Lamda); // 
	virtual double  getCenterLamda();

	virtual void	setBandWidth(double bandwidth); // Χ
	virtual double	getBandWidth();

	virtual QVector<double> getFreqList(); // ȡƵб
	virtual void setRefphaseRange(double refRange);
	virtual double getRefphaseRange();

	virtual POLARTYPEENUM getPolarType();// 
	virtual void	setPolarType(POLARTYPEENUM type);
public: // PRFFS
	virtual void setPRF(double prf); // λƵ
	virtual double getPRF();

	virtual double getFs(); // Ƶ
	virtual void setFs(double fs);

	virtual double getCenterLookAngle() ;
	virtual void setCenterLookAngle(double angle) ;
public:// ߷ͼ
	virtual void setTransformRadiationPattern(std::shared_ptr<AbstractRadiationPattern> radiationPanttern); //  ䷽ͼ
 	virtual void setReceiveRadiationPattern(std::shared_ptr<AbstractRadiationPattern> radiationPanttern); // V շͼ

	virtual std::shared_ptr<AbstractRadiationPattern> getTransformRadiationPattern();// H ䷽ͼ 
	virtual std::shared_ptr<AbstractRadiationPattern> getReceiveRadiationPattern();// V ䷽ͼ 

public:
	virtual double getPt();
	virtual double getGri();
	virtual void setPt(double Pt);
	virtual void setGri(double gri);

private:
	double refRangePhase;
	double DopplerParametersReferenceTime;
	std::vector<double> DopplerCentroidCoefficients;
	std::vector<double> DopplerRateValuesCoefficients;

public:
	virtual double getDopplerParametersReferenceTime()  ;
	virtual void   setDopplerParametersReferenceTime(double time)  ;
	virtual std::vector<double> getDopplerCentroidCoefficients()  ;
	virtual void				setDopplerCentroidCoefficients(std::vector<double> DopplerCentroids)  ;
	virtual std::vector<double> getDopplerRateValuesCoefficients()  ;
	virtual void				setDopplerRateValuesCoefficients(std::vector<double>  DopplerRateValues)  ;


};


/** ͨú ************************************************************************** */


/// <summary>
/// xmlļжȡǼڵ
/// </summary>
/// <param name="xmlPath"></param>
/// <param name="nodes"></param>
/// <returns></returns>
ErrorCode ReadSateGPSPointsXML(QString xmlPath, std::vector<SatelliteOribtNode>& nodes);
std::vector<SatelliteOribtNode> FilterSatelliteOribtNode(std::vector<SatelliteOribtNode>& nodes, double startTime, double endTime,long minCount=10);
std::shared_ptr<AbstractRadiationPattern> CreateAbstractRadiationPattern(std::vector<RadiationPatternGainPoint> antPatternPoints);

std::vector<RadiationPatternGainPoint> ReadGainFile(QString antPatternFilePath);


/** ߷ͼļչʾ ********************************************************************** */








/**  Ƶʼ ********************************************************************** */
/**
 * ο
 * ϳɿ׾״㷨ʵ֣cumming ,  ϳɿ׾ĸ
 * ״źŴMark A.Richards,ϵ
 * ϳɿ׾״ԭƤ
 * InSARԭӦã
 * SARزźŷоֽ
 * غϳɿ׾״ԭʼزģо 
 * SARزź㷨ܱȽоΤ
 * غϳɿ׾״ӰУо¶ѧ
 * ע ռ㹫ʽ  ¶ѧĶռ㹫ʽڲ죬мʡһЩ任
 * .
 */

// Ƶ
double  getDopplerCenterFreq(double &lamda,double &R,Vector3D &Rs,Vector3D &Rt,Vector3D &Vs,Vector3D &Vt);

// յƵʣбʣ
double  getDopplerFreqRate(double& lamda, double& R, Vector3D& Rs, Vector3D& Rt, Vector3D& Vs, Vector3D& Vt,Vector3D &Ast);














